/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
* Operator of Los Alamos National Laboratory.
* This file is distributed subject to a Software License Agreement found
* in the file LICENSE that is included with this distribution. 
\*************************************************************************/
package doubleLinkList;
import doubleLinkList.DLNode;

public class DLList extends Object {
    public DLList() { first=null;last=null;count=0;}
    public final int getCount() {return count;}
    public final DLNode getFirst() {return first;}
    public final DLNode getLast() {return last;}
    public final DLNode getNext(DLNode node) {
	if(node==null) throw new
          java.util.NoSuchElementException("getNext called with null argument");
	return node.next;
    }
    public final DLNode getPrev(DLNode node)
    {
	if(node==null) throw new
          java.util.NoSuchElementException("getPrev called with null argument");
        return node.prev;
    }
    public final void addEnd(DLNode node)
    {
	if(first==null) {
	    first = last = node;
	    node.next = node.prev = null;
	} else {
            node.next = null; node.prev = last; last.next = node; last = node;
	}
	count++;
    }
    public final void addBegin(DLNode node)
    {
	if(first==null) {
	    first = last = node;
	    node.next = node.prev = null;
	} else {
            node.next = first; node.prev = null; first.prev = node; first = node;
	}
	count++;
    }
    public final void addAfter(DLNode node,DLNode newnode)
    {
	if(node==null) throw new
          java.util.NoSuchElementException("addAfter called with null argument");
        newnode.next = node.next; newnode.prev = node;
        if(node.next==null) {
            last = newnode;
        } else {
            node.next.prev = newnode;
        }
        node.next = newnode;
        count++;
    }
    public final void addBefore(DLNode node,DLNode newnode)
    {
	if(node==null) throw new
          java.util.NoSuchElementException("addBefore called with null argument");
        newnode.next = node; newnode.prev = node.prev;
        if(node.prev==null) {
            first = newnode;
        } else {
            node.prev.next = newnode;
        }
        node.prev = newnode;
        count++;
    }
    public final void empty() {first=last=null; count = 0;}
    public final void removeNode(DLNode node)
    {
        if(node==null) throw new
         java.util.NoSuchElementException("removeNode called with null argument");
        if(node.prev==null) {
            first = node.next;
        } else {
            node.prev.next = node.next;
        }
        if(node.next==null) {
            last = node.prev;
        } else {
            node.next.prev = node.prev;
        }
        count--;
    }

    private DLNode first;
    private DLNode last;
    private int count;
}
